#!/bin/sh
################################################################################
# YaST pre-configuration validation tool for obtaining default Server Context
# while adding a Server to an eDirectory Tree or to a Domain
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2006-2007
# Novell, Inc.  All rights reserved.
################################################################################

# Return Values
# -------------
# 0 The Server Context is obtained and printed in stdout
# 1 Server is not reachable using the given IP address
# 2 Error in Parameter 

# Common Locations and Files
# --------------------------
LDAP_COMMAND=/opt/novell/eDirectory/bin/ldapsearch
GETCERT_COMMAND=/opt/novell/oes-install/util/getSSCert
LDAP_PORT=389
LDAPS_PORT=636
SERVER_CERT=/etc/opt/novell/certs/SSCert.der
THIS_PRG=$0

# Constants
# ---------
DomainPrefix="ou=Novell"

# Input Parsing
# -------------
ParseInputs()
{
	if [ $# -lt 6 ]
	then
		gettext -s "Usage: ndsGetServerContext.sh -t <tree name> -a <admin name> -s <ip address> [-L <ldap_port>] [-l <SSL port>]"
		exit 2
	fi

	while getopts t:a:s:L:l: OPTION;
	do
		case $OPTION in
			t) NdsTreeName=$OPTARG ;;
			a) NdsAdminName=$OPTARG ;;
			s) ExistingServerIP=$OPTARG ;;
			L) LDAP_PORT=$OPTARG ;;
			l) LDAPS_PORT=$OPTARG ;;
			*) gettext -s "Invalid option" ;;
		esac
	done


	test -z "$NdsTreeName"  && LogToErr "Missing argument: NdsTreeName"  2
	test -z "$NdsAdminName"  && LogToErr "Missing argument: NdsAdminName"  2
	test -z "$ExistingServerIP"  && LogToErr "Missing argument: ExistingServerIP"  2
	test -z "$ADM_PASSWD" && LogToErr "Missing environment variable: ADM_PASSWD" 2

	ExistingServerIP=`echo "$ExistingServerIP" |cut -f 1 -d ':'`  # if the port is different, YaST should pass the address as xx.xx.xx.xx:xx
	ExistingServerPort=`echo "$3" | cut -s -f 2 -d ':'`
	test -z "$ExistingServerPort"  && ExistingServerPort=0
	RequireSecureChannel=1

	NdsAdminContext=`echo $NdsAdminName | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/' | cut -s -f 2 -d","`
	LdapAdminName=`echo $NdsAdminName | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/g'`
	LdapAdminContext=`echo $NdsAdminContext | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/g'`
}


# Log messages to stderr
# ----------------------
LogToErr()
{
	echo 1>&2 "$THIS_PRG: $1"
	test $2 -ne 0 && exit $2
}

# Print the given arguments for debugging
# ---------------------------------------
PrintArgs()
{
	LogToErr "LDAP Based utility to retrieve server context for YaST" 0
	LogToErr " NdsTreeName : $NdsTreeName" 0
	LogToErr " NdsAdminName : $NdsAdminName" 0
	LogToErr " ExistingServerIP : $ExistingServerIP" 0
	test -n "$ExistingServerPort" && LogToErr " ExistingServerPort: $ExistingServerPort" 0
}

GetLDAPCommandString()
{
	#Test if the LDAP server is reachable and we are able to do ldapsearch.  Attempt the secure port if none is given.
	#LD_LIBRARY_PATH=/opt/novell/eDirectory/lib64:/opt/novell/eDirectory/lib64/nds-modules:$LD_LIBRARY_PATH "${GETCERT_COMMAND}" -a "${ExistingServerIP}" -t "${NdsTreeName}" -u "${NdsAdminName}" -x "${ADM_PASSWD}"> /dev/null 2>&1
	`/opt/novell/xad/share/dcinit/getsscert_script.sh $ExistingServerIP`;
	if [ ! -f "${SERVER_CERT}" ]; then
		LogToErr "Unable to retrieve the CA certificate. Secure LDAP will not be attempted..." 0
		RequireSecureChannel=0
	fi

	# first try the given port. We will not attempt any other port if this fails.
	if [ $ExistingServerPort -ne 0 ]; then
		if [ $RequireSecureChannel -eq 1 ]; then
			LDAPCommand="/usr/bin/ldapsearch -LLL -D $LdapAdminName -w $ADM_PASSWD -H ldaps://$ExistingServerIP:$ExistingServerPort "
		else
			LDAPCommand="/usr/bin/ldapsearch -x -LLL -D $LdapAdminName -w $ADM_PASSWD -H ldaps://$ExistingServerIP:$ExistingServerPort "
		fi
		LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCommand -b "" -s base > /dev/null 2>&1
		test $? -eq 0 && return 
	else
		# First attempt Secure port if certificate is available.
		if [ $RequireSecureChannel -eq 1 ]; then
			LDAPCommand="/usr/bin/ldapsearch -LLL  -D $LdapAdminName -w $ADM_PASSWD -H ldaps://$ExistingServerIP:$LDAPS_PORT "
			LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCommand -b "" -s base > /dev/null 2>&1
			test $? -eq 0 && return
		fi

		LogToErr "Unable to reach the server in secure port.  Attempting clear...." 0
		LDAPCommand="/usr/bin/ldapsearch -LLL -D $LdapAdminName -w $ADM_PASSWD -H ldaps://$ExistingServerIP:$LDAP_PORT -x  "
		LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCommand -b "" -s base > /dev/null 2>&1
		test $? -eq 0 && return
		
	fi
	LogToErr "Unable to contact or authenticate to the given server. Exiting..."  1
}

main()
{
	ParseInputs $*
	PrintArgs
	GetLDAPCommandString

	test -z "${LDAPCommand}" && exit 1

	LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCommand -E 2.16.840.1.113719.1.513.4.5 -b "${LdapAdminContext}" -s base objectclass | grep -i objectclass: | grep -i container >/dev/null
	if [ $? -eq 0 ]
	then
		# This is a Domain Services for Windows Tree
		AdminImmediateParentRDN=`echo $LdapAdminContext | awk -F"," '{ print $1 }'`
		echo "$AdminImmediateParentRDN" | grep -i "cn=users" > /dev/null
		if [ $? -eq 0 ] ; then
			NdsDomain=`echo $NdsAdminContext | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/' | cut -s -f 2 -d","`
			LdapDomain=`echo $NdsDomain | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/g'`
			if [ -z "LdapDomain" ] ; then
				NdsServerContext=${NdsAdminContext}
			else
				LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCommand -E 2.16.840.1.113719.1.513.4.5 -b "${LdapDomain}" -s base objectclass | grep -i objectclass: | grep -i domainDNS >/dev/null 
				if [ $? -eq 0 ]
				then
					NdsServerContext="${DomainPrefix}.${NdsDomain}"
				fi
			fi
		else
			NdsServerContext=${NdsAdminContext}
		fi
	else
		# This is an eDirectory Tree
		NdsServerContext=${NdsAdminContext}
	fi
	test -z "$NdsServerContext" && LogToErr "Unable to retrieve server context for YaST" 2
	LogToErr "Returning server context->$NdsServerContext" 0
	echo "$NdsServerContext"
	exit 0
}
main $*
